#include <AccelStepper.h>

AccelStepper stepperX(1, 5, 3);//mode, PUL+, DIR+
AccelStepper stepperY(1, 6, 2);
AccelStepper stepperZ(1, 7, 4);

int maxSpeed=2000;
int acceleration=1400;

int limitXBegin=10;
int limitXEnd=11;
bool resetX=false;
unsigned long limitXVal=0;

int limitYBegin=8;
int limitYEnd=9;
bool resetY=false;
unsigned long limitYVal=0;

int limitZBegin=12;
int limitZEnd=13;
bool resetZ=false;
unsigned long limitZVal=0;

bool reset=false;

bool unlocking[3]={false,false,false};

unsigned long moveToX=0;
unsigned long moveToY=0;
unsigned long moveToZ=0;

int baudRate=9600;

bool sendOnce=false;

void setup(){
  
//init serial  
  Serial.begin(baudRate);
  
//init endstop
  pinMode(limitXBegin,INPUT);
  pinMode(limitXEnd,INPUT);
  pinMode(limitYBegin,INPUT);
  pinMode(limitYEnd,INPUT);
  pinMode(limitZBegin,INPUT);
  pinMode(limitZEnd,INPUT);

//init Stepper
   stepperX.setMaxSpeed(maxSpeed);
   stepperX.setAcceleration(acceleration);
   stepperX.moveTo(moveToX);
    
   stepperY.setMaxSpeed(maxSpeed);
   stepperY.setAcceleration(acceleration);
   stepperY.moveTo(moveToY);
    
   stepperZ.setMaxSpeed(maxSpeed);
   stepperZ.setAcceleration(acceleration);
   stepperZ.moveTo(moveToZ);

}

void loop() {

  
//Listen Serial
    while (Serial.available() > 0) {
      
    // look for the next valid integer in the incoming serial stream:
    unsigned long receiveX = Serial.parseInt(); 
    // do it again:
    unsigned long receiveY = Serial.parseInt(); 
    // do it again:
    unsigned long receiveZ = Serial.parseInt(); 

    char lastC=Serial.read();
    // look for normal
    if (lastC=='n') 
    {
      moveToX = receiveX;
      moveToY = receiveY;
      moveToZ = receiveZ;
      //reset=false;
      moveToMotors();
      sendOnce=false;
    }
    // look for reset
    if (lastC=='r') {
      moveToX = receiveX+100000000; //100000000
      moveToY = receiveY+100000000;
      moveToZ = receiveZ+100000000;
      moveToMotors();
      reset=true;
      resetX=true;
      resetY=true;
      resetZ=true;
      sendOnce=false;
    }
  }
  
  
  
  if(reset==false)
  {
    normalMode();
    //LISTEN WHEN IF TASK IS OVER
    if (stepperX.distanceToGo() == 0 && stepperY.distanceToGo() == 0 && stepperZ.distanceToGo() == 0)
    {
      if(sendOnce==false){
      Serial.println("donepos");
    sendOnce=true;  
    }  
  }

  }
  else if(reset==true)
  {
    resetMode();
    //LISTEN IF EVERY AXIS REACHED RESET
    if (resetX==false && resetY==false && resetZ==false)
    {
      reset=false;
      //Serial.println("done reset");
    }
  }
  

}

void moveToMotors()
{

    stepperX.moveTo(moveToX);
    
    stepperY.moveTo(moveToY);
    
    stepperZ.moveTo(moveToZ);
    
    //send feedback to confirm data was received
    Serial.print(moveToX);
    Serial.print("x");
    Serial.print(moveToY);
    Serial.print("y");
    Serial.print(moveToZ);
    Serial.println("z");

}

void normalMode()
{

    //X   
    if(digitalRead(limitXBegin)==LOW && digitalRead(limitXEnd)==LOW){
      //if(stepperX.currentPosition()<limitXVal)
      //{
      stepperX.run();
      //}
    }
    else
    {
      //Serial.print("x:");
      //Serial.println(stepperX.currentPosition());
    }
    
    //Y
    if(digitalRead(limitYBegin)==LOW && digitalRead(limitYEnd)==LOW){
      //if(stepperY.currentPosition()<limitYVal)
      //{
      stepperY.run();
      //}
    }
    else
    {
      //Serial.print("y:");
      //Serial.println(stepperY.currentPosition());
    }
    
    //Z
    if(digitalRead(limitZBegin)==LOW && digitalRead(limitZEnd)==LOW){
      //if(stepperZ.currentPosition()<limitZVal)
      //{
      stepperZ.run();
      //}
    }
    else
    {
      //Serial.print("z:");
      //Serial.println(stepperZ.currentPosition());
    }
}

void resetMode()
{
  
     //X   
    if(digitalRead(limitXBegin)==LOW && digitalRead(limitXEnd)==LOW)
    {
      stepperX.run();
    }
    else if(digitalRead(limitXBegin)==HIGH && digitalRead(limitXEnd)==LOW && unlocking[0]==false)
   {
      unlocking[0]=true;
      limitXVal=stepperX.currentPosition();
              //Serial.print("reset x:");
        //  Serial.println(limitXVal);
    }
    if(unlocking[0]==true){
      
      unlock(1);
    }
    
    
    //Y   
    if(digitalRead(limitYBegin)==LOW && digitalRead(limitYEnd)==LOW)
    {
      stepperY.run();
    }
    else if(digitalRead(limitYBegin)==HIGH && digitalRead(limitYEnd)==LOW && unlocking[1]==false)
    {
      unlocking[1]=true;
      limitYVal=stepperY.currentPosition();
    }
    if(unlocking[1]==true){
      
      unlock(2);
    }
    
    //X   
    if(digitalRead(limitZBegin)==LOW && digitalRead(limitZEnd)==LOW){
      stepperZ.run();
    }
    else if(digitalRead(limitZBegin)==HIGH && digitalRead(limitZEnd)==LOW && unlocking[2]==false)
    {
      unlocking[2]=true;
      limitZVal=stepperZ.currentPosition();
    }
    if(unlocking[2]==true){
      
      unlock(3);
    }

}

void unlock(int state)
{
      switch (state)
      {
        case 1:
        moveToX=limitXVal-1000;
        stepperX.moveTo(moveToX);
        stepperX.run();
        if(stepperX.distanceToGo() == 0 && resetX==true ){
          Serial.print("reset x:");
          //Serial.println(limitXVal);
          Serial.println(stepperX.currentPosition());
        resetX=false;
        }
        break;
        
        case 2:
        moveToY=limitYVal-1000;
        stepperY.moveTo(moveToY);
        stepperY.run();
        if(stepperY.distanceToGo() == 0  && resetY==true){
          Serial.print("reset y:");
          //Serial.println(limitYVal);
          Serial.println(stepperY.currentPosition());
         resetY=false;
        }
        break;
        
        case 3:
        moveToZ=limitZVal-1000;
        stepperZ.moveTo(moveToZ);
        stepperZ.run();
        if(stepperZ.distanceToGo() == 0 && resetZ==true){
                Serial.print("reset z:");
                //Serial.println(limitZVal);
                Serial.println(stepperZ.currentPosition());
        resetZ=false;
        }
        
        break;
      }
}
